@@ -5,11 +5,14 @@
 */
 
 use crate::header::{
-    Pipeline, Shaders, Textures, UniformBuffer, UniformBuffers, Vertex, VertexBuffers,
+    Pipeline, Shaders, TextureFormat, TextureMode, Textures, UniformBuffer, UniformBuffers, Vertex,
+    VertexBuffers,
 };
 use image::GenericImageView;
 use wgpu::{util::DeviceExt, BindGroupLayout};
 
+// ============= PIPELINE ============= //
+
 impl crate::header::Renderer {
     /// Creates and adds the pipeline to render queue
     pub fn build_and_append_pipeline(
@@ -65,6 +68,8 @@ impl crate::header::Renderer {
     }
 }
 
+// ============= SHADERS ============= //
+
 impl crate::header::Renderer {
     /// Creates and adds the shaders to render queue
     pub fn build_and_append_shaders(
@@ -93,10 +98,13 @@ impl crate::header::Renderer {
             .create_shader_module(&wgpu::ShaderModuleDescriptor {
                 label: Some("Shader"),
                 source: wgpu::ShaderSource::Wgsl(shader_source.into()),
-                flags: wgpu::ShaderFlags::all(),
+                //flags: wgpu::ShaderFlags::all(),
             });
 
-        let mut bind_group_layouts = vec![&self.texture_bind_group_layout, &self.default_uniform_bind_group_layout];
+        let mut bind_group_layouts = vec![
+            &self.texture_bind_group_layout,
+            &self.default_uniform_bind_group_layout,
+        ];
         if uniform_layout.is_some() {
             bind_group_layouts.push(uniform_layout.unwrap())
         }
@@ -123,8 +131,8 @@ impl crate::header::Renderer {
                     module: &shader,
                     entry_point: "main",
                     targets: &[wgpu::ColorTargetState {
-                        format: self.sc_desc.format,
-                        write_mask: wgpu::ColorWrite::ALL,
+                        format: self.config.format,
+                        write_mask: wgpu::ColorWrites::ALL,
                         blend: Some(wgpu::BlendState::REPLACE),
                     }],
                 }),
@@ -132,7 +140,7 @@ impl crate::header::Renderer {
                     topology: wgpu::PrimitiveTopology::TriangleList,
                     strip_index_format: None,
                     front_face: wgpu::FrontFace::Ccw,
-                    cull_mode: None,//Some(wgpu::Face::Back),
+                    cull_mode: None, //Some(wgpu::Face::Back),
                     polygon_mode: wgpu::PolygonMode::Fill,
                     clamp_depth: false,
                     conservative: false,
@@ -167,6 +175,8 @@ impl crate::header::Renderer {
     }
 }
 
+// ============= VERTEX BUFFER ============= //
+
 impl crate::header::Renderer {
     /// Creates and adds the vertex buffers to render queue
     pub fn build_and_append_vertex_buffers(
@@ -193,7 +203,7 @@ impl crate::header::Renderer {
             .create_buffer_init(&wgpu::util::BufferInitDescriptor {
                 label: Some("Vertex Buffer"),
                 contents: bytemuck::cast_slice(verticies.as_slice()),
-                usage: wgpu::BufferUsage::VERTEX,
+                usage: wgpu::BufferUsages::VERTEX,
             });
 
         let index_buffer = self
@@ -201,7 +211,7 @@ impl crate::header::Renderer {
             .create_buffer_init(&wgpu::util::BufferInitDescriptor {
                 label: Some("Index Buffer"),
                 contents: bytemuck::cast_slice(indicies.as_slice()),
-                usage: wgpu::BufferUsage::INDEX,
+                usage: wgpu::BufferUsages::INDEX,
             });
 
         Ok(VertexBuffers {
@@ -233,6 +243,8 @@ impl crate::header::Renderer {
     }
 }
 
+// ============= UNIFORM BUFFER ============= //
+
 impl crate::header::Renderer {
     /// Creates and adds the uniform buffers to render queue
     pub fn build_and_append_uniform_buffers(
@@ -262,7 +274,7 @@ impl crate::header::Renderer {
                         &wgpu::util::BufferInitDescriptor {
                             label: Some(*name),
                             contents: bytemuck::cast_slice(&[*value]),
-                            usage: wgpu::BufferUsage::UNIFORM,
+                            usage: wgpu::BufferUsages::UNIFORM,
                         },
                     ));
                 }
@@ -271,7 +283,7 @@ impl crate::header::Renderer {
                         &wgpu::util::BufferInitDescriptor {
                             label: Some(*name),
                             contents: bytemuck::cast_slice(&[*value]),
-                            usage: wgpu::BufferUsage::UNIFORM,
+                            usage: wgpu::BufferUsages::UNIFORM,
                         },
                     ));
                 }
@@ -280,7 +292,7 @@ impl crate::header::Renderer {
                         &wgpu::util::BufferInitDescriptor {
                             label: Some(*name),
                             contents: bytemuck::cast_slice(&[*value]),
-                            usage: wgpu::BufferUsage::UNIFORM | wgpu::BufferUsage::COPY_DST,
+                            usage: wgpu::BufferUsages::UNIFORM | wgpu::BufferUsages::COPY_DST,
                         },
                     ));
                 }
@@ -298,7 +310,7 @@ impl crate::header::Renderer {
             buffer_entry.push(descriptor);
             buffer_layout.push(wgpu::BindGroupLayoutEntry {
                 binding: i as u32,
-                visibility: wgpu::ShaderStage::VERTEX | wgpu::ShaderStage::FRAGMENT,
+                visibility: wgpu::ShaderStages::VERTEX | wgpu::ShaderStages::FRAGMENT,
                 ty: wgpu::BindingType::Buffer {
                     ty: wgpu::BufferBindingType::Uniform,
                     has_dynamic_offset: false,
@@ -341,16 +353,19 @@ impl crate::header::Renderer {
     }
 }
 
+// ============= TEXTURE ============= //
+
 impl crate::header::Renderer {
     /// Creates and adds the texture to render queue
     pub fn build_and_append_texture(
         &mut self,
         name: &'static str,
-        diffuse_bytes: Vec<u8>,
-        mode: &'static str,
+        diffuse_bytes: &[u8],
+        texture_mode: TextureMode,
+        texture_format: TextureFormat,
     ) -> Result<usize, anyhow::Error> {
         let textures = self
-            .build_texture(name, diffuse_bytes, mode)
+            .build_texture(name, diffuse_bytes, texture_mode, texture_format)
             .expect("Couldn't create shaders");
         let index = self.texture_bind_group.len();
         self.texture_bind_group.push(textures);
@@ -361,20 +376,26 @@ impl crate::header::Renderer {
     pub fn build_texture(
         &mut self,
         name: &'static str,
-        diffuse_bytes: Vec<u8>,
-        mode: &'static str,
+        diffuse_bytes: &[u8],
+        texture_mode: TextureMode,
+        texture_format: TextureFormat,
     ) -> Result<Textures, ()> {
-        let _mode: wgpu::AddressMode;
-        if mode == "repeat" {
-            _mode = wgpu::AddressMode::Repeat;
-        } else if mode == "mirror_repeat" {
-            _mode = wgpu::AddressMode::MirrorRepeat;
-        } else {
-            _mode = wgpu::AddressMode::ClampToEdge;
+        let mode: wgpu::AddressMode;
+        match texture_mode {
+            TextureMode::Clamp => mode = wgpu::AddressMode::Repeat,
+            TextureMode::Repeat => mode = wgpu::AddressMode::MirrorRepeat,
+            TextureMode::MirrorRepeat => mode = wgpu::AddressMode::ClampToEdge,
+        }
+
+        let img_format = match texture_format {
+            TextureFormat::PNG => image::ImageFormat::Png,
+            TextureFormat::BMP => image::ImageFormat::Bmp,
+            TextureFormat::JPEG => image::ImageFormat::Jpeg,
+            TextureFormat::PNM => image::ImageFormat::Pnm,
         };
 
-        let img = image::load_from_memory(diffuse_bytes.as_slice())
-            .expect("Couldn't Load Image For Texture");
+        let img = image::load_from_memory_with_format(diffuse_bytes, img_format)
+            .expect(format!("Couldn't Load Image For Texture Of {}", name).as_str());
 
         let rgba = img
             .as_rgba8()
@@ -393,7 +414,7 @@ impl crate::header::Renderer {
             sample_count: 1,
             dimension: wgpu::TextureDimension::D2,
             format: wgpu::TextureFormat::Rgba8UnormSrgb,
-            usage: wgpu::TextureUsage::SAMPLED | wgpu::TextureUsage::COPY_DST,
+            usage: wgpu::TextureUsages::TEXTURE_BINDING | wgpu::TextureUsages::COPY_DST,
         });
 
         self.queue.write_texture(
@@ -401,7 +422,7 @@ impl crate::header::Renderer {
                 texture: &texture,
                 mip_level: 0,
                 origin: wgpu::Origin3d::ZERO,
-                //aspect: wgpu::TextureAspect::All,
+                aspect: wgpu::TextureAspect::All,
             },
             rgba,
             wgpu::ImageDataLayout {
@@ -414,9 +435,9 @@ impl crate::header::Renderer {
 
         let view = texture.create_view(&wgpu::TextureViewDescriptor::default());
         let sampler = self.device.create_sampler(&wgpu::SamplerDescriptor {
-            address_mode_u: _mode,
-            address_mode_v: _mode,
-            address_mode_w: _mode,
+            address_mode_u: mode,
+            address_mode_v: mode,
+            address_mode_w: mode,
             mag_filter: wgpu::FilterMode::Linear,
             min_filter: wgpu::FilterMode::Nearest,
             mipmap_filter: wgpu::FilterMode::Nearest,
