@@ -7,7 +7,9 @@
 use crate::header::{
     uniform_type, Camera, Engine, Object, Renderer, UniformBuffer, WindowDescriptor,
 };
-use crate::utils::default_resources::{DEFAULT_COLOR, DEFAULT_SHADER, DEFAULT_TEXTURE};
+use crate::utils::default_resources::{
+    DEFAULT_COLOR, DEFAULT_MATRIX_4, DEFAULT_SHADER, DEFAULT_TEXTURE,
+};
 use winit::{
     event::{Event, WindowEvent},
     event_loop::{ControlFlow, EventLoop},
@@ -22,13 +24,13 @@ impl Engine {
         env_logger::init();
         // Dimentions of the window, as width and height
         // and then are set as a logical size that the window can accept
-        let dimention = winit::dpi::LogicalSize {
+        let dimention = winit::dpi::PhysicalSize {
             width: settings.width,   // Which sets the width of the window
             height: settings.height, // And sets the height of the window
         };
 
         // Here the size is finally made according to the dimentions we set earlier
-        let size = winit::dpi::Size::Logical(dimention);
+        let size = winit::dpi::Size::Physical(dimention);
 
         // And we will create a new window and set all the options we stored
         let new_window = WindowBuilder::new()
@@ -51,15 +53,24 @@ impl Engine {
 
         let _ = renderer.build_and_append_texture(
             "Default Texture",
-            Vec::from(DEFAULT_TEXTURE),
-            "clamp",
+            DEFAULT_TEXTURE,
+            crate::header::TextureMode::Clamp,
+            crate::header::TextureFormat::PNG
         )?;
 
-        let _ = renderer
+        let _ = renderer.build_and_append_uniform_buffers(vec![UniformBuffer::Matrix(
+            "Camera Uniform",
+            camera.camera_uniform_buffer()?,
+        )])?;
+
+        let default_uniform = renderer
             .build_and_append_uniform_buffers(vec![
-                UniformBuffer::Matrix("Camera Uniform", camera.camera_uniform_buffer()?),
+                UniformBuffer::Matrix(
+                    "Transformation Matrix",
+                    uniform_type::Matrix::from_glm(DEFAULT_MATRIX_4),
+                ),
                 UniformBuffer::Array(
-                    "Default Color",
+                    "Color",
                     uniform_type::Array {
                         data: DEFAULT_COLOR,
                     },
@@ -70,7 +81,7 @@ impl Engine {
         let _ = renderer.build_and_append_shaders(
             "Default Shader",
             DEFAULT_SHADER.to_string(),
-            None,
+            Some(&default_uniform),
         )?;
 
         Ok(Self {
@@ -82,6 +93,11 @@ impl Engine {
         })
     }
 
+    /// Runs the block of code that you pass to it every frame. The update code is used
+    /// to modify the engine on the fly thus creating interactive graphics and making things
+    /// happy in the engine!
+    ///
+    /// Renderer, window, vec of objects, events, and camera are passed to the update code.
     #[allow(unreachable_code)]
     pub fn update_loop<F>(self, mut update_function: F) -> anyhow::Result<()>
     where
@@ -96,47 +112,46 @@ impl Engine {
             mut camera,
         } = self;
 
-        // Run the callback of before renderer start
-        //logic(&mut renderer, WindowCallbackEvents::Before, &window);
         // and get input events to handle them later
         let mut input = winit_input_helper::WinitInputHelper::new();
+        let mut current_window_size = window.inner_size();
+
         // The main loop
         event_loop.run(move |event, _, control_flow| {
+            // updates the data on what events happened before the frame start
             input.update(&event);
             match event {
                 Event::WindowEvent {
                     ref event,
                     window_id,
-                } if window_id == window.id() => {
-                    match event {
-                        WindowEvent::CloseRequested => *control_flow = ControlFlow::Exit,
-                        WindowEvent::Resized(physical_renderer) => {
-                            renderer.resize(*physical_renderer);
-                        }
-                        WindowEvent::ScaleFactorChanged { new_inner_size, .. } => {
-                            // new_inner_renderer is &&mut so we have to dereference it twice
-                            renderer.resize(**new_inner_size);
-                        }
-                        _ => {}
-                    }
-                }
+                } if window_id == window.id() => match event {
+                    WindowEvent::CloseRequested => *control_flow = ControlFlow::Exit,
+                    _ => {}
+                },
 
                 Event::MainEventsCleared => {
+                    let new_window_size = window.inner_size();
+                    if new_window_size != current_window_size {
+                        renderer.resize(new_window_size);
+                        current_window_size = new_window_size;
+                    }
+
                     update_function(&mut renderer, &window, &mut objects, &input, &mut camera);
-                    camera.update_view_projection(&mut renderer).expect("Couldn't update camera");
+                    camera
+                        .update_view_projection(&mut renderer)
+                        .expect("Couldn't update camera");
                     objects.iter_mut().for_each(|i| {
                         if i.changed {
-                            i.update(&mut renderer, window.inner_size())
-                                .expect("Couldn't update objects");
+                            i.update(&mut renderer).expect("Couldn't update objects");
                         }
                     });
 
                     match renderer.render() {
                         Ok(_) => {}
                         // Recreate the swap_chain if lost
-                        Err(wgpu::SwapChainError::Lost) => renderer.resize(renderer.size),
+                        Err(wgpu::SurfaceError::Lost) => renderer.resize(renderer.size),
                         // The system is out of memory, we should probably quit
-                        Err(wgpu::SwapChainError::OutOfMemory) => *control_flow = ControlFlow::Exit,
+                        Err(wgpu::SurfaceError::OutOfMemory) => *control_flow = ControlFlow::Exit,
                         // All other errors (Outdated, Timeout) should be resolved by the next frame
                         Err(e) => eprintln!("{:?}", e),
                     }
